--[[

   Graphics Demo For RPi

--]]


local lume  	= require("lume")
local class 	= require("30log")

local rnd 		= math.random
local sqrt 		= math.sqrt
local pi   		= math.pi
local sin  		= math.sin
local cos			= math.cos
local floor 	= math.floor

math.randomseed( os.time() )						-- random seed to start with

local x,y,z		= 0,0,0						-- generic x,y,z scratch pad

-- Declarations for PiFace code

local N 				= 300						-- set the number of dots per frame

local dots			= {}

local pi_tex 		= 0					-- main context for pi background image
local piface		= 0					-- and 512 x 512 canvas for drawing on
local font1			= 0 				-- better font for display

local pi_hit   	= 0					-- numerator accumulator
local pi_throw 	= 0	    		-- denominator accumulator
local pi_est		= 0         -- decimal esimate of pi



-- Declarations for Logo code

local L 				= 2000					-- set the number of pixels in the logo swarm

local XA  			= 0.015					-- rotation angles of pixel swarm
local YA  			= 0.014
local ZA  			= 0.0198

local SWARM							= 2								-- dot size

local Logo_W, Logo_H		= 100, 126
local X_OFF, Y_OFF			= 880, 400

local map3d 					= {}							-- x,y,z      for each pixel in swarm
local swarm						= {}							-- x,y,r,g,b  point table (derived from map3d)
local r,g,b						= 0,0,0
local xx,yy,zz        = 0,0,0         	-- tempory random swarm position






--[[
Vec3 is a prototype for 3d vector class. x,y,z are [1],[2],[3] in array
--]]

local Vec3 = class("Vec3", {0, 0, 0})			-- basic vector3 proto

function Vec3:init(x,y,z)
	self[1] = x  
	self[2] = y 
	self[3] = z
end

function Vec3:len()								-- radial dist from origin
	return sqrt(self[1]^2 + self[2]^2 + self[3]^2)
end

function Vec3:zrot(a)							-- rotate around the z axis
	local xp = self[1]*cos(a) - self[2]*sin(a)
	local yp = self[1]*sin(a) + self[2]*cos(a)
	self[1]  = xp
	self[2]  = yp
end
	
function Vec3:xrot(a)							-- rotate around the x axis
	local yp = self[2]*cos(a) - self[3]*sin(a)
	local zp = self[2]*sin(a) + self[3]*cos(a)
	self[2]  = yp
	self[3]  = zp
end

function Vec3:yrot(a)						-- rotate around the y axis
	local xp = self[1]*cos(a) - self[3]*sin(a)
	local zp = self[1]*sin(a) + self[3]*cos(a)
	self[1]  = xp
	self[3]  = zp
end

local function v3dot(a,b)					-- requires vec3 parameters a.b-> scalar
	return  (a[1]*b[1]  + a[2]*b[2] + a[3]*b[3])
end

local function v3dif(a,b,c)				-- requires vec3 parameters a-b-> c
	c[1] = a[1] - b[1]
	c[2] = a[2] - b[3]
	c[3] = a[3] - b[3]
end

local function v3sum(a,b,c)				-- requires vec3 parameters a+b-> c
	c[1] = a[1] + b[1]
	c[2] = a[2] + b[2]
	c[3] = a[3] + b[3]
end

local function v3dist(a,b)				-- dist between 2 vectors
	return  sqrt( (a[1]-b[1])^2 + (a[2]-b[2])^2 + (a[3]-b[3])^2)
end


function love.load()

  love.graphics.setBackgroundColor(80, 80, 80)
	
	love.graphics.setLineWidth(1)
	love.graphics.setLineJoin("none")
	
	pi_tex 	=  love.graphics.newImage( "pi.png" )		    -- image of pi background
	
	
	piface =  love.graphics.newCanvas( 512, 512 ) 			-- render to surface before draw
	love.graphics.setCanvas(piface)       							-- point to the off screen canvas
	love.graphics.clear( 50, 50, 50 )										-- and set grey
	
	for i=1,N do													-- instantiate a persistent dot array
		dots[i] = {0, 0, 250, 250, 20}			-- rgb is 250 250 20 (yellow)
	end
	
	local logo =  love.image.newImageData( "rpi-logo.png" )		-- image of pi logo
	
	for i=1,L do													-- instantiate L pixels like logo pattern
		repeat 
			xx = rnd(0,Logo_W-1)
			yy = rnd(0,Logo_H-1)
		  r,g,b = logo:getPixel(xx, yy)			-- pick a random pixel in the logo data
		until r+g+b < 520										-- background is white 255+255+255
		
		map3d[i] 			= Vec3()							-- set up the map array for swarm pixels
		map3d[i][1] 	= xx - Logo_W/2
		map3d[i][2]		= yy - Logo_H/2
		map3d[i][3]		= rnd(-10, 10)				-- randomly place in z axis to look pretty
		
		swarm[i]			= {0,0, r,g,b}				-- and set the color values of the point array
		
	end
	
	font1 = love.graphics.newFont( "Nunito-Regular.ttf", 17 )
	love.graphics.setFont(font1)
	
end

function love.keypressed(k)
	
		if k == 'escape' then
      love.event.quit()
		end
end


function love.update(dt)
	
	-- PiFace calulations
	for i=1,N do
		x = rnd()*2 - 1    								-- find a random number between -1 and +1 for x
    y = rnd()*2 - 1    								-- find a random number between -1 and +1 for y
		dots[i][1] = x * 254 + 256				-- fill in the dots arrays
		dots[i][2] = y * 254 + 256				-- 
		dots[i][3] = 50										-- assume miss - green
		
		pi_throw = pi_throw + 1         	-- always advance the "throw" accumulator by 1
		if (x*x + y*y) < 1 then 					-- if point inside the circle,
			pi_hit = pi_hit + 4     				-- advance the "hit" accumulator by 4
			dots[i][3] = 250								-- hit - turn red on to get yellow
		end
	end
	pi_est = pi_hit/pi_throw
	
	-- LogoSwarm calculations
	
	for i = 1,L do											-- rotate the map of pixels in 3 axes
		map3d[i]:xrot(XA)
		map3d[i]:yrot(YA)
		map3d[i]:zrot(ZA)
		
		x = map3d[i][1] * 2.0								-- scale the vectors out to a 100r cube
		y = map3d[i][2] * 2.0
		z = map3d[i][3] * 2.0 + 400 				  -- move z out from origin
		
		swarm[i][1] = x * 400/z	+ X_OFF				-- perspective scaling and txlate
		swarm[i][2] = y * 400/z	+ Y_OFF
	end
	
end

function love.draw()
	
	love.graphics.setCanvas(piface)       				-- work on the off screen canvas
	love.graphics.setColor(255,255,255,18)				-- phosphor effect
	love.graphics.draw(pi_tex)										-- fade out old image
		
	love.graphics.setColor(255,255,255,255)				-- draw the dots
	love.graphics.setPointSize(2)
	love.graphics.points(dots)
	
	
	love.graphics.setCanvas()       							-- pointing back to screen
	love.graphics.setColor(200,200,200)
  love.graphics.draw(piface,120,150,pi/0.5,1,1)
	
	love.graphics.setColor(255,255,255)
	love.graphics.setPointSize(SWARM)
	love.graphics.points(swarm)
	
	love.graphics.setColor(252,252,252)
	love.graphics.print("Num: "..pi_hit, 120,670)
	love.graphics.print("Denom: "..pi_throw, 278,670)
	love.graphics.print("Pi: "..pi_est, 465,670)
	love.graphics.print("FPS: " ..love.timer.getFPS( ), 1100,20)

end

