
--[[
     Iterative geometric algorithm for estimating pi. 
		 
		A unit circle (radius 1, area pi) is inscribed in a square (side 2, area 4)
		Random points in the square are calculated and tested for inclusion in the circle.
		Any point with a distance < 1 is inside the circle. (x^2 + y^2) < 1 avoids sqrt.
		From geometry, the chance of a random point being in the circle is pi/4 (area of circle/area of square)
		A point in the circle is a "hit", an iteration is a "throw". Accumulate 
		the number of throws and the number of hits, then multiply ratio by 4 to get pi ...
		
		pi/4 =  hits/throws
		pi   =  4 x  hits/throws
--]]


N = 10000000

print("\n -----",N," geometric estimates of pi   ----- \n")

math.randomseed( os.time() )
local rand = math.random

local pi_hit   = 0		-- numerator accumulator
local pi_throw = 0	    -- denominator accumulator
local pi =0             -- decimal esimate of pi

local x                 -- tempory random x  -1 <--> +1
local y                 -- tempory random y  -1 <--> +1


for  n=1, N do
	
    x = rand()*2 - 1    -- find a random number between -1 and +1 for x
    y = rand()*2 - 1    -- find a random number between -1 and +1 for y
		
	if (x*x + y*y) < 1 then pi_hit = pi_hit + 1  end   -- if point inside the circle, advance the "hit" accumulator by 4
	pi_throw = pi_throw + 1                            -- always advance the "throw" accumulator by 1
		
end

pi = pi_hit/pi_throw * 4

print(pi_hit, "div by", pi_throw, "x 4 ------>  ", pi, "    error% ---->  ", pi/math.pi*100 - 100,"%")

-- here we could build a message with the values "throw" and "hit" and send them to another node for further accumulation
-- By varying N, we can set the rate messages are generated.



